@@ -21,10 +21,10 @@ class EltakoWsSensor(Thing):
 
         self.gpio_number = gpio_number
         self.start_time = time.time()
-        self.imp = 0
+        self.num_raise_events = 0
         GPIO.setmode(GPIO.BCM)
         GPIO.setup(self.gpio_number, GPIO.IN)
-        GPIO.add_event_detect(self.gpio_number, GPIO.BOTH, callback=self.__spin, bouncetime=5)
+        GPIO.add_event_detect(self.gpio_number, GPIO.RISING, callback=self.__spin, bouncetime=5)
 
         self.timer = tornado.ioloop.PeriodicCallback(self.__measure, 10000)
 
@@ -44,26 +44,34 @@ class EltakoWsSensor(Thing):
         self.timer.start()
 
     def __spin(self, channel):
-        self.imp = self.imp + 1
+        self.num_raise_events = self.num_raise_events + 1
 
     def __measure(self):
-        elapsed_sec = time.time() - self.start_time
-        windspeed_kmh = self.__compute_speed_kmh(self.imp, elapsed_sec)
-        self.imp = 0
-        self.start_time = time.time()
-        logging.info('windspeed ' + str(windspeed_kmh))
-        self.windspeed.notify_of_external_update(windspeed_kmh)
+        try:
+            windspeed_kmh = 0
+            elapsed_sec = time.time() - self.start_time
+            if (self.num_raise_events > 0) and (elapsed_sec > 0):
+                windspeed_kmh = self.__compute_speed_kmh(self.num_raise_events, elapsed_sec)
+            self.num_raise_events = 0
+            self.start_time = time.time()
+            logging.info('windspeed ' + str(windspeed_kmh) + " (" + str(self.num_raise_events) + ' raise events/' + str(elapsed_sec) + ' sec)')
+            self.windspeed.notify_of_external_update(windspeed_kmh)
+        except Exception as e:
+            logging.debug(str(e))
 
-    def __compute_speed_kmh(self, imp, elapsed_sec):
-        rotation_per_sec = (imp / elapsed_sec) / 2
-        km_per_hour = 1.761 / (1 + rotation_per_sec) + 3.813 * rotation_per_sec
-        if km_per_hour < 1:
+    def __compute_speed_kmh(self, num_raise_events, elapsed_sec):
+        rotation_per_sec = num_raise_events / elapsed_sec
+        lowspeed_factor = 1.761
+        highspeed_factor = 3.013
+        km_per_hour = lowspeed_factor / (1 + rotation_per_sec) + highspeed_factor * rotation_per_sec
+        if km_per_hour < 2:
             km_per_hour = 0
         return round(km_per_hour, 1)
 
-    def cancel_update_level_task(self):
+    def cancel_measure_task(self):
         self.timer.stop()
 
+
 def run_server(port, gpio_number, description):
     eltakows_sensor = EltakoWsSensor(gpio_number, description)
     server = WebThingServer(SingleThing(eltakows_sensor), port=port)
@@ -71,9 +79,8 @@ def run_server(port, gpio_number, description):
         logging.info('starting the server')
         server.start()
     except KeyboardInterrupt:
-        logging.debug('canceling the sensor update looping task')
-        eltakows_sensor.cancel_update_level_task()
         logging.info('stopping the server')
+        eltakows_sensor.cancel_measure_task()
         server.stop()
         logging.info('done')
 
